@@ -1,11 +1,13 @@
 use crate::core::atom::Atom;
 use crate::core::block::Block;
+use crate::core::check_error::{CheckError, OpenConnection, OpenMap};
 use crate::core::named_path::NamedPath;
 use crate::core::probe::Probe;
 
 #[derive(Default)]
 struct CheckConnected {
     path: NamedPath,
+    failures: OpenMap,
 }
 
 impl Probe for CheckConnected {
@@ -19,17 +21,8 @@ impl Probe for CheckConnected {
 
     fn visit_atom(&mut self, name: &str, signal: &dyn Atom) {
         if !signal.connected() {
-            panic!(
-                "Signal #{} {}::{} has no driver!",
-                signal.id(),
-                self.path.to_string(),
-                name
-            );
+            self.failures.insert(signal.id(), OpenConnection{ path: self.path.to_string(), name: name.to_string() });
         }
-        /* TODO - add logging
-        else {
-            println!("Signal {}::{} is driven", self.path.to_string(), name);
-        }*/
     }
 
     fn visit_end_namespace(&mut self, _name: &str, _node: &dyn Block) {
@@ -41,7 +34,12 @@ impl Probe for CheckConnected {
     }
 }
 
-pub fn check_connected(uut: &dyn Block) {
+pub fn check_connected(uut: &dyn Block) -> Result<(), CheckError> {
     let mut visitor = CheckConnected::default();
     uut.accept("uut", &mut visitor);
+    if visitor.failures.is_empty() {
+        Ok(())
+    } else {
+        Err(CheckError::OpenSignal(visitor.failures))
+    }
 }
