@@ -114,7 +114,7 @@ impl Logic for AD7193Simulator {
     #[hdl_gen]
     fn update(&mut self) {
         // Connect the spi bus
-        self.wires.link(&mut self.spi_slave.wires);
+        SPIWiresSlave::link(&mut self.wires, &mut self.spi_slave.wires);
         // Clock internal components
         self.reg_ram.read_clock.next = self.clock.val();
         self.reg_ram.write_clock.next = self.clock.val();
