@@ -1,8 +1,7 @@
 use rust_hdl::core::prelude::*;
 use rust_hdl::hls::prelude::*;
-use rust_hdl::widgets::prelude::*;
 
-#[derive(LogicBlock, Default)]
+#[derive(LogicBlock)]
 struct MOSIPortTest {
     bus: SoCBusController<16, 2>,
     bridge: Bridge<16, 2, 2>,
@@ -11,6 +10,18 @@ struct MOSIPortTest {
     clock: Signal<In, Clock>,
 }
 
+impl Default for MOSIPortTest {
+    fn default() -> Self {
+        Self {
+            bus: Default::default(),
+            bridge: Bridge::new(["port_a", "port_b"]),
+            port_a: Default::default(),
+            port_b: Default::default(),
+            clock: Default::default(),
+        }
+    }
+}
+
 impl Logic for MOSIPortTest {
     #[hdl_gen]
     fn update(&mut self) {
@@ -145,7 +156,7 @@ fn test_port_pipeline() {
     .unwrap();
 }
 
-#[derive(LogicBlock, Default)]
+#[derive(LogicBlock)]
 struct MOSIWidePortTest {
     bus: SoCBusController<16, 2>,
     bridge: Bridge<16, 2, 2>,
@@ -154,6 +165,24 @@ struct MOSIWidePortTest {
     clock: Signal<In, Clock>,
 }
 
+impl Default for MOSIWidePortTest {
+    fn default() -> Self {
+        Self {
+            bus: Default::default(),
+            bridge: Bridge::new(["port_a", "port_b"]),
+            port_a: Default::default(),
+            port_b: Default::default(),
+            clock: Default::default(),
+        }
+    }
+}
+
+impl HLSNamedPorts for MOSIWidePortTest {
+    fn ports(&self) -> Vec<String> {
+        self.bridge.ports()
+    }
+}
+
 impl Logic for MOSIWidePortTest {
     #[hdl_gen]
     fn update(&mut self) {
@@ -246,7 +275,7 @@ fn test_wide_port_test_works() {
     .unwrap();
 }
 
-#[derive(LogicBlock, Default)]
+#[derive(LogicBlock)]
 struct MOSIPortFIFOTest {
     bus: SoCBusController<16, 2>,
     bridge: Bridge<16, 2, 1>,
@@ -254,11 +283,28 @@ struct MOSIPortFIFOTest {
     clock: Signal<In, Clock>,
 }
 
+impl Default for MOSIPortFIFOTest {
+    fn default() -> Self {
+        Self {
+            bus: Default::default(),
+            bridge: Bridge::new(["port_a"]),
+            port_a: Default::default(),
+            clock: Default::default(),
+        }
+    }
+}
+
+impl HLSNamedPorts for MOSIPortFIFOTest {
+    fn ports(&self) -> Vec<String> {
+        self.bridge.ports()
+    }
+}
+
 impl Logic for MOSIPortFIFOTest {
     #[hdl_gen]
     fn update(&mut self) {
         self.bus.clock.next = self.clock.val();
-        SoCBusController::<16,2>::join(&mut self.bus, &mut self.bridge.upstream);
+        SoCBusController::<16, 2>::join(&mut self.bus, &mut self.bridge.upstream);
         SoCPortController::<16>::join(&mut self.bridge.nodes[0], &mut self.port_a.bus);
     }
 }
